searchData={"items":[{"type":"module","title":"gen_tailscale","doc":"Interface to TCP/IP sockets over a [Tailscale](https://tailscale.com/) network.\n\nThis module provides\n[`gen_tcp`](https://www.erlang.org/doc/apps/kernel/gen_tcp.html)-like\nfunctionality for accepting connections over a\n[Tailscale](https://tailscale.com/) network using\n[libtailscale](https://github.com/tailscale/libtailscale/).\n\n> #### Warning {: .warning}\n>\n> This is a rough proof-of-concept. It uses crudely modified versions of the\n> original `gen_tcp` and\n> [`gen_tcp_socket`](https://github.com/erlang/otp/blob/master/lib/kernel/src/gen_tcp_socket.erl)\n> modules to manage the tcp-sockets and should probably not be used for anything\n> important as is.\n>\n> Most of the documentation in here is left-over from `gen_tcp` and cannot be\n> counted on to work as advertised.","ref":"gen_tailscale.html"},{"type":"function","title":"gen_tailscale.accept/1","doc":"Equivalent to  accept(ListenSocket, infinity) .","ref":"gen_tailscale.html#accept/1"},{"type":"function","title":"gen_tailscale.accept/2","doc":"Accept an incoming connection request on a listen socket.\n\n`Socket` must be a socket returned from `listen/2`. `Timeout` specifies\na time-out value in milliseconds. Defaults to `infinity`.\n\nReturns:\n\n- `{ok, Socket}` if a connection is established\n- `{error, closed}` if `ListenSocket` is closed\n- `{error, timeout}` if no connection is established within `Timeout`\n- `{error, system_limit}` if all available ports in the Erlang emulator\n  are in  use\n- A POSIX error value if something else goes wrong, see `m:inet`\n  about possible values\n\nTo send packets (outbound) on the returned `Socket`, use `send/2`.\nPackets sent from the peer (inbound) are delivered as messages\nto the socket owner; the process that created the socket.\nUnless `{active, false}` is specified in the option list when creating\nthe [listening socket](`listen/2`).\n\nSee `connect/4` about _active mode_ socket messages and _passive mode_.\n\n> #### Note {: .info }\n>\n> The `accept` call _doesn't have to be_ issued from the socket owner process.\n> Using version 5.5.3 and higher of the emulator, multiple simultaneous accept\n> calls can be issued from different processes, which allows for a pool of\n> acceptor processes handling incoming connections.","ref":"gen_tailscale.html#accept/2"},{"type":"function","title":"gen_tailscale.close/1","doc":"Close a TCP socket.\n\nNote that in most implementations of TCP, doing a `close` does not guarantee\nthat the data sent is delivered to the recipient.  It is guaranteed that\nthe recepient will see all sent data before getting the close, but the\nsender gets no indication of that.\n\nIf the sender needs to know that the recepient has received all data\nthere are two common ways to achieve this:\n\n1. Use [`gen_tailscale:shutdown(Sock, write)`](`shutdown/2`) to signal that no more\n   data is to be sent and wait for the other side to acknowledge seeing\n   its read side being closed, by closing its write side, which shows\n   as a socket close on this side.\n2. Implement an acknowledgement in the protocol on top of TCP\n   that both connection ends adhere to, indicating that all data\n   has been seen.  The socket option [`{packet, N}`](`m:inet#option-packet`)\n   may be useful.","ref":"gen_tailscale.html#close/1"},{"type":"function","title":"gen_tailscale.connect/2","doc":"Equivalent to [`connect(SockAddr, Opts, infinity)`](`connect/3`).","ref":"gen_tailscale.html#connect/2"},{"type":"function","title":"gen_tailscale.connect/3","doc":"Create a socket connected to the specified address.","ref":"gen_tailscale.html#connect/3"},{"type":"function","title":"With arguments Address and Port - gen_tailscale.connect/3","doc":"Equivalent to [`connect(Address, Port, Opts, infinity)`](`connect/4`).","ref":"gen_tailscale.html#connect/3-with-arguments-address-and-port"},{"type":"function","title":"With argument SockAddr (since OTP 24.3) - gen_tailscale.connect/3","doc":"Connects to a remote listen socket specified by `SockAddr`\nwhere `t:socket:sockaddr_in6/0` for example allows specifying\nthe `scope_id` for link local IPv6 addresses.\n\n[IPv4 addresses](`t:socket:sockaddr_in/0`) on the same\n`t:map/0` format is also allowed.\n\nEquivalent to `connect/4`, besides the format of the destination address.","ref":"gen_tailscale.html#connect/3-with-argument-sockaddr-since-otp-24-3"},{"type":"function","title":"gen_tailscale.connect/4","doc":"Create a socket connected to the specified address.\n\nCreates a socket and connects it to a server on TCP port `Port`\non the host with IP address `Address`, that may also be a hostname.\n\n### `Opts` (connect options)\n\n- **`{ip, Address}`** - If the local host has many IP addresses,\n  this option specifies which one to use.\n\n- **`{ifaddr, Address}`** - Same as `{ip, Address}`.\n\n  However, if `Address` instead is a `t:socket:sockaddr_in/0` or\n  `t:socket:sockaddr_in6/0` this takes precedence over any value\n  previously set with the `ip` and `port` options. If these options\n  (`ip` or/and `port`) however comes _after_ this option,\n  they may be used to _update_ the corresponding fields of this option\n  (for `ip`, the `addr` field, and for `port`, the `port` field).\n\n- **`{fd, integer() >= 0}`** - If a socket has somehow been connected without\n  using `gen_tailscale`, use this option to pass the file descriptor for it.\n  If `{ip, Address}` and/or `{port, port_number()}` is combined\n  with this option, the `fd` is bound to the specified interface\n  and port before connecting. If these options are not specified,\n  it is assumed that the `fd` is already bound appropriately.\n\n- **`inet`** - Sets up the socket for IPv4.\n\n- **`inet6`** - Sets up the socket for IPv6.\n\n- **`local`** - Sets up a Unix Domain Socket. See `t:inet:local_address/0`\n\n- **`{port, Port}`** - Specifies which local port number to use.\n\n- **`{tcp_module, module()}`** - Overrides which callback module is used.\n  Defaults to `inet_tcp` for IPv4 and `inet6_tcp` for IPv6.\n\n- **`t:option/0`** - See `inet:setopts/2`.","ref":"gen_tailscale.html#connect/4"},{"type":"function","title":"Socket Data - gen_tailscale.connect/4","doc":"Packets can be sent to the peer (outbound) with\n[`send(Socket, Packet)`](`send/2`).  Packets sent from the peer\n(inbound) are delivered as messages to the socket owner;\nthe process that created the socket, unless `{active, false}`\nis specified in the `Options` list.\n\n#### Active mode socket messages\n\n- **`{tcp, Socket, Data}`** - Inbound data from the socket.\n\n- **`{tcp_passive, Socket}`** -\n  The socket was in `{active, N}` mode (see `inet:setopts/2` for details)\n  and its message counter reached `0`, indicating that\n  the socket has transitioned to passive (`{active, false}`) mode.\n\n\n- **`{tcp_closed, Socket}`** - The socket was closed.\n\n- **`{tcp_error, Socket, Reason}`** A socket error occurred.\n\n#### Passive mode\n\nIf `{active, false}` is specified in the option list for the socket,\npackets and errors are retrieved by calling [`recv/2,3`](`recv/3`)\n(`send/2` may also return errors).\n\n#### Timeout\n\nThe optional `Timeout` parameter specifies a connect time-out in milliseconds.\nDefaults to `infinity`.\n\n> #### Note {: .info }\n>\n> Keep in mind that if the underlying OS `connect()` call returns a timeout,\n> `gen_tailscale:connect` will also return a timeout (i.e. `{error, etimedout}`),\n> even if a larger `Timeout` was specified (for example `infinity`).\n\n> #### Note {: .info }\n>\n> The default values for options specified to `connect` can be affected by the\n> Kernel configuration parameter `inet_default_connect_options`.\n> For details, see `m:inet`.","ref":"gen_tailscale.html#connect/4-socket-data"},{"type":"type","title":"gen_tailscale.connect_option/0","doc":"","ref":"gen_tailscale.html#t:connect_option/0"},{"type":"function","title":"gen_tailscale.controlling_process/2","doc":"Change the controlling process (owner) of a socket.\n\nAssigns a new controlling process `Pid` to `Socket`. The controlling process\nis the process that the socket sends messages to.  If this function\nis called from any other process than the current controlling process,\n`{error, not_owner}` is returned.\n\nIf the process identified by `Pid` is not an existing local `t:pid/0`,\n`{error, badarg}` is returned. `{error, badarg}` may also be returned\nin some cases when `Socket` is closed during the execution of this function.\n\nIf the socket is in _active mode_, this function will transfer any messages\nfrom the socket in the mailbox of the caller to the new controlling process.\n\nIf any other process is interacting with the socket during the transfer,\nit may not work correctly and messages may remain in the caller's mailbox.\nFor instance, changing the sockets active mode during the transfer\ncould cause this.","ref":"gen_tailscale.html#controlling_process/2"},{"type":"function","title":"gen_tailscale.listen/2","doc":"Create a listen socket.\n\nCreates a socket and sets it to listen on port `Port` on the local host.\n\nIf `Port == 0`, the underlying OS assigns an available (ephemeral)\nport number, use `inet:port/1` to retrieve it.\n\nThe following options are available:\n\n- **`list`** - Received `Packet`s are delivered as lists of bytes,\n  `[`[`byte/0`](`t:byte/0`)`]`.\n\n- **`binary`** - Received `Packet`s are delivered as `t:binary/0`s.\n\n- **`{backlog, B}`** - `B ::` `t:non_neg_integer/0`. The backlog value\n  defines the maximum length that the queue of pending connections\n  can grow to. Defaults to `5`.\n\n- **`inet6`** - Sets up the socket for IPv6.\n\n- **`inet`** - Sets up the socket for IPv4.\n\n- **`{fd, Fd}`** - If a socket has somehow been created without using\n  `gen_tailscale`, use this option to pass the file descriptor for it.\n\n- **`{ip, Address}`** - If the host has many IP addresses, this option\n  specifies which one to listen on.\n\n- **`{port, Port}`** - Specifies which local port number to use.\n\n- **`{ifaddr, Address}`** - Same as `{ip, Address}`.\n\n  However, if this instead is an `t:socket:sockaddr_in/0` or\n  `t:socket:sockaddr_in6/0` this takes precedence over any value\n  previously set with the `ip` and `port` options. If these options\n  (`ip` or/and `port`) however comes _after_ this option,\n  they may be used to _update_ their corresponding fields of this option\n  (for `ip`, the `addr` field, and for `port`, the `port` field).\n\n- **`{tcp_module, module()}`** - Overrides which callback module is used.\n  Defaults to `inet_tcp` for IPv4 and `inet6_tcp` for IPv6.\n\n- **`t:option/0`** - See `inet:setopts/2`.\n\nThe returned socket `ListenSocket` should be used when calling\n[`accept/1,2`](`accept/1`) to accept an incoming connection request.\n\n> #### Note {: .info }\n>\n> The default values for options specified to `listen` can be affected by the\n> Kernel configuration parameter `inet_default_listen_options`. For details, see\n> `m:inet`.","ref":"gen_tailscale.html#listen/2"},{"type":"type","title":"gen_tailscale.listen_option/0","doc":"","ref":"gen_tailscale.html#t:listen_option/0"},{"type":"type","title":"gen_tailscale.option/0","doc":"","ref":"gen_tailscale.html#t:option/0"},{"type":"type","title":"gen_tailscale.option_name/0","doc":"","ref":"gen_tailscale.html#t:option_name/0"},{"type":"type","title":"gen_tailscale.pktoptions_value/0","doc":"Value from socket option [`pktoptions`](`t:option_name/0`).\n\nIf the platform implements the IPv4 option `IP_PKTOPTIONS`,\nor the IPv6 option `IPV6_PKTOPTIONS` or `IPV6_2292PKTOPTIONS` for the socket;\nthis value is returned from `inet:getopts/2` when called with the option name\n[`pktoptions`](`t:option_name/0`).\n\n> #### Note {: .info }\n>\n> This option appears to be VERY Linux specific, and its existence in future\n> Linux kernel versions is also worrying since the option is part of RFC 2292\n> which is since long (2003) obsoleted by RFC 3542 that _explicitly_ removes\n> this possibility to get packet information from a stream socket. For\n> comparison: it has existed in FreeBSD but is now removed, at least since\n> FreeBSD 10.","ref":"gen_tailscale.html#t:pktoptions_value/0"},{"type":"function","title":"gen_tailscale.recv/2","doc":"Equivalent to  recv(Socket, Length, infinity) .","ref":"gen_tailscale.html#recv/2"},{"type":"function","title":"gen_tailscale.recv/3","doc":"Receive a packet, from a socket in _passive mode_.\n\nA closed socket is indicated by the return value `{error, closed}`.\nIf the socket is not in passive mode, the return value is `{error, einval}`.\n\nArgument `Length` is only meaningful when the socket is in `raw` mode and\ndenotes the number of bytes to read.  If `Length` is `0`, all available\nbytes are returned. If `Length > 0`, exactly `Length` bytes are returned,\nor an error; except if the socket is closed from the other side,\nthen the last read before the one returning `{error, closed}`\nmay return less than `Length` bytes of data.\n\nThe optional `Timeout` parameter specifies a time-out in milliseconds.\nDefaults to `infinity`.\n\nAny process can receive data from a passive socket, even if that process is not\nthe controlling process of the socket. However, only one process can call this\nfunction on a socket at any given time. Using simultaneous calls to `recv` is\nnot recommended as the behavior depends on the socket implementation,\nand could return errors such as `{error, ealready}`.","ref":"gen_tailscale.html#recv/3"},{"type":"function","title":"gen_tailscale.send/2","doc":"Send a packet on a socket.\n\nThere is no `send/2` call with a time-out option; use socket option\n`send_timeout` if time-outs are desired.  See section\n[Examples](#module-examples).\n\nThe return value `{error, {timeout, RestData}}` can only be returned when\n`inet_backend = socket`.\n\n[](){: #non_blocking_send }\n\n> #### Note {: .info }\n>\n> #### Non-blocking send.\n>\n> If the user tries to send more data than there is room for in the OS send\n> buffers, the 'rest data' is stored in (inet driver) internal buffers and later\n> sent in the background. The function immediately returns ok (_not_ informing\n> the caller that some date isn'nt sent yet). Any issue while\n> sending the 'rest data' may be returned later.\n>\n> When using `inet_backend = socket`, the behaviour is different. There is\n> _no_ buffering, instead the caller will \"hang\" until all of the data\n> has been sent or the send timeout (as specified by the `send_timeout`\n> option) expires (the function can \"hang\" even when using the `inet`\n> backend if the internal buffers are full).\n>\n> If this happens when using `packet =/= raw`, a partial packet has been\n> written. A new packet therefore _mustn't_ be written at this point,\n> as there is no way for the peer to distinguish this from data in\n> the current packet. Instead, set the `packet` option to `raw`, send the\n> rest data (as raw data) and then set `packet` back to the correct type.","ref":"gen_tailscale.html#send/2"},{"type":"function","title":"gen_tailscale.shutdown/2","doc":"Close the socket in one or both directions.\n\n`How == write` means closing the socket for writing, reading from it is still\npossible.\n\nIf `How == read` or there is no outgoing data buffered in the `Socket` port, the\nshutdown is performed immediately and any error encountered is returned in\n`Reason`.\n\nIf there is data buffered in the socket port, shutdown isn't performed\non the socket until that buffered data has been written to the OS\nprotocol stack.  If any errors are encountered, the socket is closed\nand `{error, closed}` is returned by the next `recv/2` or `send/2` call.\n\nOption `{exit_on_close, false}` is useful if the peer performs a shutdown\nof its write side.  Then the socket stays open for writing after\nreceive has indicated that the socket was closed.\n\n[](){: #async_shutdown_write }\n\n> #### Note {: .info }\n>\n> Async shutdown write (`How :: write | read_write`).\n>\n> If the shutdown attempt is made while the inet driver is sending\n> buffered data in the background, the shutdown is postponed until\n> all buffered data has been sent.  This function immediately returns `ok`,\n> and the caller _isn't_ informed (that the shutdown has been postponed).\n>\n> When using `inet_backend = socket`, the behaviour is different. A shutdown\n> with `How :: write | read_write` will always be performed _immediately_.","ref":"gen_tailscale.html#shutdown/2"},{"type":"type","title":"gen_tailscale.socket/0","doc":"As returned by [`accept/1,2`](`accept/1`) and [`connect/3,4`](`connect/3`).","ref":"gen_tailscale.html#t:socket/0"},{"type":"module","title":"gen_tailscale_socket","doc":"Low-level interface to the sockets generated from `m:gen_tailscale`.\n\n> #### Warning {: .warning}\n>\n> This is a rough proof-of-concept. It uses a crudely modified version of the\n> original [`gen_tcp_socket`](https://github.com/erlang/otp/blob/master/lib/kernel/src/gen_tcp_socket.erl)\n> module to manage the tcp-sockets and should probably not be used for anything\n> important as is.\n>\n> Most of the documentation in here is left-over from `gen_tcp_socket` and cannot be\n> counted on to work as advertised.","ref":"gen_tailscale_socket.html"},{"type":"function","title":"gen_tailscale_socket.accept/2","doc":"","ref":"gen_tailscale_socket.html#accept/2"},{"type":"function","title":"gen_tailscale_socket.callback_mode/0","doc":"","ref":"gen_tailscale_socket.html#callback_mode/0"},{"type":"function","title":"gen_tailscale_socket.cancel_monitor/1","doc":"","ref":"gen_tailscale_socket.html#cancel_monitor/1"},{"type":"function","title":"gen_tailscale_socket.close/1","doc":"","ref":"gen_tailscale_socket.html#close/1"},{"type":"function","title":"gen_tailscale_socket.connect/3","doc":"","ref":"gen_tailscale_socket.html#connect/3"},{"type":"function","title":"gen_tailscale_socket.connect/4","doc":"","ref":"gen_tailscale_socket.html#connect/4"},{"type":"function","title":"gen_tailscale_socket.controlling_process/2","doc":"","ref":"gen_tailscale_socket.html#controlling_process/2"},{"type":"function","title":"gen_tailscale_socket.fdopen/2","doc":"","ref":"gen_tailscale_socket.html#fdopen/2"},{"type":"function","title":"gen_tailscale_socket.getopts/2","doc":"","ref":"gen_tailscale_socket.html#getopts/2"},{"type":"function","title":"gen_tailscale_socket.getremoteaddr/1","doc":"","ref":"gen_tailscale_socket.html#getremoteaddr/1"},{"type":"function","title":"gen_tailscale_socket.getstat/2","doc":"","ref":"gen_tailscale_socket.html#getstat/2"},{"type":"function","title":"gen_tailscale_socket.handle_event/4","doc":"","ref":"gen_tailscale_socket.html#handle_event/4"},{"type":"function","title":"gen_tailscale_socket.info/1","doc":"","ref":"gen_tailscale_socket.html#info/1"},{"type":"function","title":"gen_tailscale_socket.init/1","doc":"","ref":"gen_tailscale_socket.html#init/1"},{"type":"function","title":"gen_tailscale_socket.listen/2","doc":"","ref":"gen_tailscale_socket.html#listen/2"},{"type":"function","title":"gen_tailscale_socket.monitor/1","doc":"","ref":"gen_tailscale_socket.html#monitor/1"},{"type":"function","title":"gen_tailscale_socket.peername/1","doc":"","ref":"gen_tailscale_socket.html#peername/1"},{"type":"function","title":"gen_tailscale_socket.recv/3","doc":"","ref":"gen_tailscale_socket.html#recv/3"},{"type":"function","title":"gen_tailscale_socket.send/2","doc":"","ref":"gen_tailscale_socket.html#send/2"},{"type":"function","title":"gen_tailscale_socket.sendfile/4","doc":"","ref":"gen_tailscale_socket.html#sendfile/4"},{"type":"function","title":"gen_tailscale_socket.setopts/2","doc":"","ref":"gen_tailscale_socket.html#setopts/2"},{"type":"function","title":"gen_tailscale_socket.shutdown/2","doc":"","ref":"gen_tailscale_socket.html#shutdown/2"},{"type":"function","title":"gen_tailscale_socket.socket_setopts/2","doc":"","ref":"gen_tailscale_socket.html#socket_setopts/2"},{"type":"function","title":"gen_tailscale_socket.socket_to_list/1","doc":"","ref":"gen_tailscale_socket.html#socket_to_list/1"},{"type":"function","title":"gen_tailscale_socket.sockname/1","doc":"","ref":"gen_tailscale_socket.html#sockname/1"},{"type":"function","title":"gen_tailscale_socket.socknames/1","doc":"","ref":"gen_tailscale_socket.html#socknames/1"},{"type":"function","title":"gen_tailscale_socket.start_loopback/1","doc":"","ref":"gen_tailscale_socket.html#start_loopback/1"},{"type":"function","title":"gen_tailscale_socket.terminate/3","doc":"","ref":"gen_tailscale_socket.html#terminate/3"},{"type":"function","title":"gen_tailscale_socket.unrecv/2","doc":"","ref":"gen_tailscale_socket.html#unrecv/2"},{"type":"function","title":"gen_tailscale_socket.which_packet_type/1","doc":"","ref":"gen_tailscale_socket.html#which_packet_type/1"},{"type":"function","title":"gen_tailscale_socket.which_sockets/0","doc":"","ref":"gen_tailscale_socket.html#which_sockets/0"},{"type":"extras","title":"gen_tailscale","doc":"# gen_tailscale\n\nThis module provides\n[`gen_tcp`](https://www.erlang.org/doc/apps/kernel/gen_tcp.html)-like\nfunctionality for accepting connections over a\n[Tailscale](https://tailscale.com/) network using\n[libtailscale](https://github.com/tailscale/libtailscale/).\n\nThis module is based in the [`libtailscale` NIF\nwrapper](https://hex.pm/packages/libtailscale) and is published at on Hex as\n[`gen_tailscale`](https://hex.pm/packages/gen_tailscale). I've also released\n[`tailscale_transport`](https://hex.pm/packages/tailscale_transport), which\nallows users to expose their bandit/phoenix-based app directly to their tailnet\nusing `libtailscale` and `gen_tailscale` wrapper.\n\nEverything in this chain of packages should be considered proof of concept at\nthis point and should not be used for anything important. Especially\n`gen_tailscale`, which has been constructed by crudely hacking the original\n`gen_tcp` module to use `libtailscale` and could use a total rewrite at some\npoint. However, it works well enough that my example application\n[`tschat`](https://github.com/Munksgaard/tschat) is able to accept connections\nfrom different Tailscale users and show their username by retrieving data from\nthe Tailscale connection.\n\n> #### Warning {: .warning}\n>\n> This is a rough proof-of-concept. It uses crudely modified versions of the\n> original `gen_tcp` and\n> [`gen_tcp_socket`](https://github.com/erlang/otp/blob/master/lib/kernel/src/gen_tcp_socket.erl)\n> modules to manage the tcp-sockets and should probably not be used for anything\n> important as is.","ref":"readme.html"},{"type":"extras","title":"Examples - gen_tailscale","doc":"Start a simple echo server by running `mix run examples/echo.exs` or the following code:\n\n```elixir\n{:ok, lsock} =\n  :gen_tailscale.listen(2000, active: false, hostname: \"gen-tailscale-test\", ephemeral: true)\n\n{:ok, sock} = :gen_tailscale.accept(lsock)\n\nIO.puts(\"Accepted connection from: #{inspect(:gen_tailscale_socket.getremoteaddr(sock))}\")\n\n{:ok, s} = :gen_tailscale.recv(sock, 0)\n:ok = :gen_tailscale.send(sock, s)\n:ok = :gen_tailscale.shutdown(sock, :read_write)\n:ok = :gen_tailscale.close(sock)\n:ok = :gen_tailscale.close(lsock)\n```\n\nIt will ask you to log in by following a link.\n\nOnce logged in, connect from another terminal by running `telnet\ngen-tailscale-test 2000`. You can now send a single message and have it echoed\nback to you before the server closes.","ref":"readme.html#examples"},{"type":"extras","title":"Apache License","doc":"# Apache License\nVersion 2.0, January 2004\n\nhttp://www.apache.org/licenses/\n\nTERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION","ref":"license.html"},{"type":"extras","title":"1. Definitions. - Apache License","doc":"\"License\" shall mean the terms and conditions for use, reproduction, and\ndistribution as defined by Sections 1 through 9 of this document.\n\n\"Licensor\" shall mean the copyright owner or entity authorized by the copyright\nowner that is granting the License.\n\n\"Legal Entity\" shall mean the union of the acting entity and all other entities\nthat control, are controlled by, or are under common control with that entity.\nFor the purposes of this definition, \"control\" means (i) the power, direct or\nindirect, to cause the direction or management of such entity, whether by\ncontract or otherwise, or (ii) ownership of fifty percent (50%) or more of the\noutstanding shares, or (iii) beneficial ownership of such entity.\n\n\"You\" (or \"Your\") shall mean an individual or Legal Entity exercising\npermissions granted by this License.\n\n\"Source\" form shall mean the preferred form for making modifications, including\nbut not limited to software source code, documentation source, and configuration\nfiles.\n\n\"Object\" form shall mean any form resulting from mechanical transformation or\ntranslation of a Source form, including but not limited to compiled object code,\ngenerated documentation, and conversions to other media types.\n\n\"Work\" shall mean the work of authorship, whether in Source or Object form, made\navailable under the License, as indicated by a copyright notice that is included\nin or attached to the work (an example is provided in the Appendix below).\n\n\"Derivative Works\" shall mean any work, whether in Source or Object form, that\nis based on (or derived from) the Work and for which the editorial revisions,\nannotations, elaborations, or other modifications represent, as a whole, an\noriginal work of authorship. For the purposes of this License, Derivative Works\nshall not include works that remain separable from, or merely link (or bind by\nname) to the interfaces of, the Work and Derivative Works thereof.\n\n\"Contribution\" shall mean any work of authorship, including the original version\nof the Work and any modifications or additions to that Work or Derivative Works\nthereof, that is intentionally submitted to Licensor for inclusion in the Work\nby the copyright owner or by an individual or Legal Entity authorized to submit\non behalf of the copyright owner. For the purposes of this definition,\n\"submitted\" means any form of electronic, verbal, or written communication sent\nto the Licensor or its representatives, including but not limited to\ncommunication on electronic mailing lists, source code control systems, and\nissue tracking systems that are managed by, or on behalf of, the Licensor for\nthe purpose of discussing and improving the Work, but excluding communication\nthat is conspicuously marked or otherwise designated in writing by the copyright\nowner as \"Not a Contribution.\"\n\n\"Contributor\" shall mean Licensor and any individual or Legal Entity on behalf\nof whom a Contribution has been received by Licensor and subsequently\nincorporated within the Work.","ref":"license.html#1-definitions"},{"type":"extras","title":"2. Grant of Copyright License. - Apache License","doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable copyright license to reproduce, prepare Derivative Works of,\npublicly display, publicly perform, sublicense, and distribute the Work and such\nDerivative Works in Source or Object form.","ref":"license.html#2-grant-of-copyright-license"},{"type":"extras","title":"3. Grant of Patent License. - Apache License","doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable (except as stated in this section) patent license to make, have\nmade, use, offer to sell, sell, import, and otherwise transfer the Work, where\nsuch license applies only to those patent claims licensable by such Contributor\nthat are necessarily infringed by their Contribution(s) alone or by combination\nof their Contribution(s) with the Work to which such Contribution(s) was\nsubmitted. If You institute patent litigation against any entity (including a\ncross-claim or counterclaim in a lawsuit) alleging that the Work or a\nContribution incorporated within the Work constitutes direct or contributory\npatent infringement, then any patent licenses granted to You under this License\nfor that Work shall terminate as of the date such litigation is filed.","ref":"license.html#3-grant-of-patent-license"},{"type":"extras","title":"4. Redistribution. - Apache License","doc":"You may reproduce and distribute copies of the Work or Derivative Works thereof\nin any medium, with or without modifications, and in Source or Object form,\nprovided that You meet the following conditions:\n\n1. You must give any other recipients of the Work or Derivative Works a copy of\n   this License; and\n\n2. You must cause any modified files to carry prominent notices stating that\n   You changed the files; and\n\n3. You must retain, in the Source form of any Derivative Works that You\n   distribute, all copyright, patent, trademark, and attribution notices from\n   the Source form of the Work, excluding those notices that do not pertain to\n   any part of the Derivative Works; and\n\n4. If the Work includes a \"NOTICE\" text file as part of its distribution, then\n   any Derivative Works that You distribute must include a readable copy of the\n   attribution notices contained within such NOTICE file, excluding those\n   notices that do not pertain to any part of the Derivative Works, in at least\n   one of the following places: within a NOTICE text file distributed as part\n   of the Derivative Works; within the Source form or documentation, if\n   provided along with the Derivative Works; or, within a display generated by\n   the Derivative Works, if and wherever such third-party notices normally\n   appear. The contents of the NOTICE file are for informational purposes only\n   and do not modify the License. You may add Your own attribution notices\n   within Derivative Works that You distribute, alongside or as an addendum to\n   the NOTICE text from the Work, provided that such additional attribution\n   notices cannot be construed as modifying the License.\n\nYou may add Your own copyright statement to Your modifications and may provide\nadditional or different license terms and conditions for use, reproduction, or\ndistribution of Your modifications, or for any such Derivative Works as a whole,\nprovided Your use, reproduction, and distribution of the Work otherwise complies\nwith the conditions stated in this License.","ref":"license.html#4-redistribution"},{"type":"extras","title":"5. Submission of Contributions. - Apache License","doc":"Unless You explicitly state otherwise, any Contribution intentionally submitted\nfor inclusion in the Work by You to the Licensor shall be under the terms and\nconditions of this License, without any additional terms or conditions.\nNotwithstanding the above, nothing herein shall supersede or modify the terms of\nany separate license agreement you may have executed with Licensor regarding\nsuch Contributions.","ref":"license.html#5-submission-of-contributions"},{"type":"extras","title":"6. Trademarks. - Apache License","doc":"This License does not grant permission to use the trade names, trademarks,\nservice marks, or product names of the Licensor, except as required for\nreasonable and customary use in describing the origin of the Work and\nreproducing the content of the NOTICE file.","ref":"license.html#6-trademarks"},{"type":"extras","title":"7. Disclaimer of Warranty. - Apache License","doc":"Unless required by applicable law or agreed to in writing, Licensor provides the\nWork (and each Contributor provides its Contributions) on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied,\nincluding, without limitation, any warranties or conditions of TITLE, NON-\nINFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are\nsolely responsible for determining the appropriateness of using or\nredistributing the Work and assume any risks associated with Your exercise of\npermissions under this License.","ref":"license.html#7-disclaimer-of-warranty"},{"type":"extras","title":"8. Limitation of Liability. - Apache License","doc":"In no event and under no legal theory, whether in tort (including negligence),\ncontract, or otherwise, unless required by applicable law (such as deliberate\nand grossly negligent acts) or agreed to in writing, shall any Contributor be\nliable to You for damages, including any direct, indirect, special, incidental,\nor consequential damages of any character arising as a result of this License or\nout of the use or inability to use the Work (including but not limited to\ndamages for loss of goodwill, work stoppage, computer failure or malfunction, or\nany and all other commercial damages or losses), even if such Contributor has\nbeen advised of the possibility of such damages.","ref":"license.html#8-limitation-of-liability"},{"type":"extras","title":"9. Accepting Warranty or Additional Liability. - Apache License","doc":"While redistributing the Work or Derivative Works thereof, You may choose to\noffer, and charge a fee for, acceptance of support, warranty, indemnity, or\nother liability obligations and/or rights consistent with this License. However,\nin accepting such obligations, You may act only on Your own behalf and on Your\nsole responsibility, not on behalf of any other Contributor, and only if You\nagree to indemnify, defend, and hold each Contributor harmless for any liability\nincurred by, or claims asserted against, such Contributor by reason of your\naccepting any such warranty or additional liability.\n\nEND OF TERMS AND CONDITIONS\n\nCopyright 2025, Philip Munksgaard  .\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\nhttp://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"license.html#9-accepting-warranty-or-additional-liability"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"GenTailscale v0.1.0 - Changelog","doc":"This is the initial release. It contains support for listening, accepting,\nreading and sending on Tailscale sockets using a very crudely modified version\nof `gen_tcp`.","ref":"changelog.html#gentailscale-v0-1-0"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}